@@ -1,6 +1,6 @@
 #!/usr/bin/python3
 
-from gi.repository import Gdk, GObject, GLib, Gio
+from gi.repository import Gdk, GObject, GLib, Gio, CScreensaver
 import time
 import traceback
 import os
@@ -15,7 +15,6 @@ from stage import Stage
 import singletons
 from util import utils, settings, trackers
 from util.focusNavigator import FocusNavigator
-from util.grabHelper import GrabHelper
 
 class ScreensaverManager(GObject.Object):
     """
@@ -40,7 +39,6 @@ class ScreensaverManager(GObject.Object):
         status.Locked = False
         status.Awake = False
 
-        self.grab_helper = GrabHelper(self)
         self.focus_nav = FocusNavigator()
 
         self.session_client = singletons.SessionClient
@@ -110,6 +108,8 @@ class ScreensaverManager(GObject.Object):
             - destroying the screensaver stage.
             - releasing our keyboard and mouse grabs.
         """
+        self.grab_helper = CScreensaver.EventGrabber.new(status.Debug)
+
         if active:
             if not status.Active:
                 self.cinnamon_client.exit_expo_and_overview()
@@ -129,7 +129,7 @@ class ScreensaverManager(GObject.Object):
                 return True
         else:
             if self.stage:
-                self.despawn_stage(self.on_despawn_stage_complete)
+                self.despawn_stage()
                 Gio.Application.get_default().release()
                 status.focusWidgets = []
             self.grab_helper.release()
@@ -171,7 +171,10 @@ class ScreensaverManager(GObject.Object):
             return
 
         if status.Debug and not status.Awake:
-            print("manager: user activity, waking")
+            print("*******************************************")
+            print("manager: user activity, waking:")
+            # traceback.print_stack()
+            print("*******************************************")
 
         if status.Locked and self.stage.initialize_pam():
             if status.Debug and not status.Awake:
@@ -318,13 +321,6 @@ class ScreensaverManager(GObject.Object):
         self.fb_failed_to_start = False
         self.fb_pid = 0
 
-    def despawn_stage(self, callback=None):
-        """
-        Begin destruction of the stage.
-        """
-        self.stage.cancel_unlocking()
-        self.stage.deactivate(callback)
-
     def on_spawn_stage_complete(self):
         """
         Called after the stage become visible.  All user events are now
@@ -333,6 +329,7 @@ class ScreensaverManager(GObject.Object):
         by our ConsoleKit client if we're using it, and our own ScreensaverService.)
         """
         self.grab_stage()
+        self.stage.connect("needs-refresh", self.refresh_stage)
 
         status.Active = True
 
@@ -340,11 +337,14 @@ class ScreensaverManager(GObject.Object):
 
         self.start_timers()
 
-    def on_despawn_stage_complete(self):
+    def despawn_stage(self):
         """
-        Called after the stage has been hidden - the stage is destroyed, our status
+        The stage is destroyed, our status
         is updated, timer is canceled and active-changed is fired.
         """
+        self.stage.cancel_unlocking()
+        self.stage.hide()
+
         was_active = status.Active == True
         status.Active = False
 
@@ -357,10 +357,8 @@ class ScreensaverManager(GObject.Object):
         self.stage = None
 
         # Ideal time to check for leaking connections that might prevent GC by python and gobject
-        if trackers.DEBUG_SIGNALS:
+        if status.Debug:
             trackers.con_tracker_get().dump_connections_list()
-
-        if trackers.DEBUG_TIMERS:
             trackers.timer_tracker_get().dump_timer_list()
 
     def grab_stage(self):
@@ -369,9 +367,21 @@ class ScreensaverManager(GObject.Object):
         by us now.
         """
         if self.stage != None:
-            self.grab_helper.move_to_window(self.stage.get_window(), True)
+            self.grab_helper.move_to_window(self.stage.get_window(), Gdk.Screen.get_default(),  True)
 
-    def update_stage(self):
+    def stage_refreshed(self):
+        if status.Debug:
+            print("manager: stage refresh complete")
+
+        self.grab_stage()
+
+        if status.Locked:
+            self.kill_fallback_window()
+            self.spawn_fallback_window()
+
+        self.simulate_user_activity()
+
+    def refresh_stage(self, stage=None):
         """
         Tells the stage to check its canvas size and make sure its windows are up-to-date.  This is called
         when our login manager tells us its "Active" property has changed.  We are always connected to the
@@ -381,8 +391,10 @@ class ScreensaverManager(GObject.Object):
             return
 
         if status.Debug:
-            print("manager: queuing stage refresh (login manager reported active?")
+            print("manager: queuing stage refresh")
 
+        self.kill_fallback_window()
+        self.cancel_unlocking()
         self.stage.queue_refresh_stage()
 
     def start_timers(self):
@@ -400,7 +412,7 @@ class ScreensaverManager(GObject.Object):
         self.activated_timestamp = 0
         self.stop_lock_delay()
 
-    def cancel_unlock_widget(self):
+    def cancel_unlocking(self):
         """
         Return to sleep (not Awake) - hides the pointer and the unlock widget.
         """
